@@ -1,4 +1,5 @@
-use crate::utils::{deserialize_u256, deserialize_u64};
+use crate::utils::{deserialize_optional_h160, deserialize_u256, deserialize_u64};
+use chrono::{DateTime, Utc};
 use ethers_core::{
     types::{transaction::response::Transaction, Address, Bytes, TxHash, H256, U256, U64},
     utils::keccak256,
@@ -242,8 +243,12 @@ pub struct SimulatedTransaction {
     #[serde(rename = "fromAddress")]
     pub from: Address,
     /// The destination of this transaction.
+    ///
+    /// If this is `None`, then the transaction was to a newly
+    /// deployed contract.
     #[serde(rename = "toAddress")]
-    pub to: Address,
+    #[serde(deserialize_with = "deserialize_optional_h160")]
+    pub to: Option<Address>,
     /// The value sent in this transaction.
     #[serde(deserialize_with = "deserialize_u256")]
     pub value: U256,
@@ -290,6 +295,28 @@ pub struct SimulatedBundle {
     pub transactions: Vec<SimulatedTransaction>,
 }
 
+/// Represents stats for a submitted bundle.
+///
+/// See [Flashbots docs][fb_getbundlestats] for more information.
+///
+/// [fb_getbundlestats]: https://docs.flashbots.net/flashbots-auction/searchers/advanced/rpc-endpoint/#flashbots_getbundlestats
+#[derive(Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct BundleStats {
+    /// Whether the bundle was simulated.
+    pub is_simulated: bool,
+    /// Whether the bundle was sent to miners.
+    pub is_sent_to_miners: bool,
+    /// Whether the bundle is high priority.
+    pub is_high_priority: bool,
+    /// When the bundle was simulated
+    pub simulated_at: Option<DateTime<Utc>>,
+    /// When the bundle was submitted
+    pub submitted_at: Option<DateTime<Utc>>,
+    /// When the bundle was sent to miners
+    pub sent_to_miners_at: Option<DateTime<Utc>>,
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -343,8 +370,19 @@ mod tests {
         "gasUsed": 21000,
         "toAddress": "0x73625f59CAdc5009Cb458B751b3E7b6b48C06f2C",
         "txHash": "0xa839ee83465657cac01adc1d50d96c1b586ed498120a84a64749c0034b4f19fa",
+        "value": "0x1"
+      },
+      {
+        "coinbaseDiff": "10000000000063000",
+        "ethSentToCoinbase": "10000000000000000",
+        "fromAddress": "0x02A727155aeF8609c9f7F2179b2a1f560B39F5A0",
+        "gasFees": "63000",
+        "gasPrice": "476190476193",
+        "gasUsed": 21000,
+        "toAddress": "0x",
+        "txHash": "0xa839ee83465657cac01adc1d50d96c1b586ed498120a84a64749c0034b4f19fa",
         "value": "0x"
-      }
+      } 
     ],
     "stateBlockNumber": 5221585,
     "totalGasUsed": 42000
@@ -369,11 +407,44 @@ mod tests {
         assert_eq!(simulated_bundle.gas_used, U256::from(42000));
         assert_eq!(simulated_bundle.gas_fees, U256::from(126000));
         assert_eq!(simulated_bundle.simulation_block, U64::from(5221585));
-        assert_eq!(simulated_bundle.transactions.len(), 2);
+        assert_eq!(simulated_bundle.transactions.len(), 3);
+        assert_eq!(simulated_bundle.transactions[0].value, U256::zero());
         assert_eq!(
             simulated_bundle.transactions[0].error,
             Some("execution reverted".into())
         );
         assert_eq!(simulated_bundle.transactions[1].error, None);
+        assert_eq!(simulated_bundle.transactions[1].value, U256::from(0x1));
+        assert_eq!(simulated_bundle.transactions[2].to, None);
+    }
+
+    #[test]
+    fn bundle_stats_deserialize() {
+        let bundle_stats: BundleStats = serde_json::from_str(
+            r#"{
+  "isSimulated": true,
+  "isSentToMiners": true,
+  "isHighPriority": true,
+  "simulatedAt": "2021-08-06T21:36:06.317Z",
+  "submittedAt": "2021-08-06T21:36:06.250Z",
+  "sentToMinersAt": "2021-08-06T21:36:06.343Z"
+}"#,
+        )
+        .unwrap();
+
+        assert_eq!(bundle_stats.is_simulated, true);
+        assert_eq!(bundle_stats.is_sent_to_miners, true);
+        assert_eq!(
+            bundle_stats.simulated_at.unwrap().to_rfc3339(),
+            "2021-08-06T21:36:06.317+00:00"
+        );
+        assert_eq!(
+            bundle_stats.submitted_at.unwrap().to_rfc3339(),
+            "2021-08-06T21:36:06.250+00:00"
+        );
+        assert_eq!(
+            bundle_stats.sent_to_miners_at.unwrap().to_rfc3339(),
+            "2021-08-06T21:36:06.343+00:00"
+        );
     }
 }
