@@ -1,11 +1,12 @@
 use crate::{
-    bundle::{BundleRequest, SimulatedBundle},
+    bundle::{BundleHash, BundleRequest, BundleStats, SimulatedBundle},
     pending_bundle::PendingBundle,
-    relay::{Relay, RelayError, SendBundleResponse},
+    relay::{GetBundleStatsParams, GetUserStatsParams, Relay, RelayError, SendBundleResponse},
+    UserStats,
 };
 use async_trait::async_trait;
 use ethers_core::{
-    types::{BlockNumber, Bytes},
+    types::{BlockNumber, Bytes, U64},
     utils::keccak256,
 };
 use ethers_providers::{FromErr, Middleware, PendingTransaction};
@@ -145,6 +146,46 @@ impl<M: Middleware, S: Signer> FlashbotsMiddleware<M, S> {
             self.provider(),
         ))
     }
+
+    /// Get stats for a particular bundle.
+    pub async fn get_bundle_stats(
+        &self,
+        bundle_hash: BundleHash,
+        block_number: U64,
+    ) -> Result<BundleStats, FlashbotsMiddlewareError<M, S>> {
+        self.relay
+            .request(
+                "flashbots_getBundleStats",
+                [GetBundleStatsParams {
+                    bundle_hash,
+                    block_number,
+                }],
+            )
+            .await
+            .map_err(FlashbotsMiddlewareError::RelayError)
+    }
+
+    /// Get stats for your searcher identity.
+    ///
+    /// Your searcher identity is determined by the signer you
+    /// constructed the middleware with.
+    pub async fn get_user_stats(&self) -> Result<UserStats, FlashbotsMiddlewareError<M, S>> {
+        let latest_block = self
+            .inner
+            .get_block_number()
+            .await
+            .map_err(FlashbotsMiddlewareError::MiddlewareError)?;
+
+        self.relay
+            .request(
+                "flashbots_getUserStats",
+                [GetUserStatsParams {
+                    block_number: latest_block,
+                }],
+            )
+            .await
+            .map_err(FlashbotsMiddlewareError::RelayError)
+    }
 }
 
 #[async_trait]
